<?php

/**
 * API pour l'interrogation d'Opens Datas.
 *
 * Cette API interroge les Opens Datas pertinents
 * pour y trouver les contours des territoires.
 * Elle peuple la table spip_gis de la forme géométrique,
 * établit le lien avec l'objet territoire dans spip_gis_liens
 * et garde une trace de cette opération.
 *
 * API à utiliser :
 *     ```
 *     include_spip('inc/contour');
 *     ```
 *
 * @plugin     Contours
 *
 * @copyright  2019-2020
 * @author     Vincent CALLIES
 * @licence    GNU/GPL
 *
 * @package    SPIP\Contours\inc
 */

// Sécurité
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_CONTOUR_URL_BASE_ISOCODE')) {
	define('_CONTOUR_URL_BASE_ISOCODE', 'https://contrib.spip.net/http.api/ezrest');
}


/**
 * Acquiert les données de contours en provenance de l'API REST de Nomenclatures.
 * Contours utilise un cache du plugin Cache Factory afin de limiter les flux échangés avec le serveur Nomenclatures
 * sachant que les données sont volumineuses et changent peu.
 *
 * @param string $type
 * @param string $pays
 * @param array  $service
 *
 * @return array
 */
function contours_acquerir($type, $pays, $service) {

	// Initialisation du tableau de sortie.
	$contours = array();

	// Lecture du cache si il existe et test de sa validité
	include_spip('inc/ezcache_cache');
	$cache = array(
		'type'    => $type,
		'pays'    => $pays,
		'service' => $service
	);
	if (!$fichier_cache = cache_est_valide('contours', 'geodata', $cache)) {
		// Construire les filtres permettant ensuite de calculer l'url de la requête
		$filtres = array();
		if ($type) {
			$filtres['type'] = $type;
		}
		if ($pays) {
			$filtres['pays'] = $pays;
		}
		if ($service) {
			$filtres['service'] = $service;
		}

		// Ajouter les exclusions si nécessaire
		if (!empty($options['exclure'])) {
			$filtres['exclure'] = $options['exclure'];
		}

		// Appel à l'API REST de Nomenclatures
		$requeter = charger_fonction('requeter_isocode', 'inc');
		if ($data = $requeter(_CONTOUR_URL_BASE_ISOCODE, 'contours', $filtres)) {
			// On extrait uniquement l'index des données.
			$contours = $data['donnees'];

			// Mise à jour du cache correspondant
			cache_ecrire('contours', 'geodata', $cache, json_encode($contours));
		}

	} else {
		// Lecture des données du fichier cache valide
		$contours = cache_lire('contours', 'geodata', $fichier_cache);
	}

	return $contours;
}


/**
 * @param mixed  $type_service
 *
 * @return array
 */
function services_geometrie_acquerir($type_service) {

	// Initialisation du tableau de sortie.
	$services = array();

	// Construire les filtres permettant ensuite de calculer l'url de la requête
	$filtres = array(
		'type_service' => $type_service,
	);

	// Appel à l'API REST de Nomenclatures
	$requeter = charger_fonction('requeter_isocode', 'inc');
	if ($data = $requeter(_CONTOUR_URL_BASE_ISOCODE, 'services', $filtres)) {
		$services = $data['donnees'][$type_service];
	}

	return $services;
}


/**
 * Identifie la variable de configuration à partir du type de territoire, du pays et du service éventuellement.
 *
 * @api
 *
 * @param string $type    Type de territoires à peupler. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays dont on veut peupler les subdivisions.
 *                        N'est utilisé que si le type choisi est `subdivision`.
 * @param string $service Identifiant du service Nomenclatures ayant permis le chargement. Est utilisé pour indexer
 *                        la consignation dans le cas des contours uniquement. Sinon vaut chaine vide
 *
 * @return string
 */
function consigne_contour_identifier($type, $pays, $service) {

	return ($type === 'subdivision')
		? "${type}/${pays}/${service}"
		: "${type}/${service}";
}
